@@ -4,11 +4,11 @@
 # @license      http://opensource.org/licenses/MIT MIT
 # @trademark    HUBzero is a registered trademark of The Regents of the University of California.
 #
-__version__ = '0.3.6'
+__version__ = '0.4.1'
 
 from .utils import getGetSimToolNameRevisionFromEnvironment, findInstalledSimToolNotebooks, searchForSimTool
 from .utils import findInstalledSimToolNotebooks as findSimTools
-from .utils import parse, getValidatedInputs
+from .utils import parse, getValidatedInputs, getParamsFromDictionary
 from .utils import findSimToolNotebook, getSimToolInputs, getSimToolOutputs
 from .run import Run, DB 
 from .experiment import Experiment, set_experiment, get_experiment
